#!/usr/bin/python

import sys
from PyQt4 import QtCore, QtGui, QtOpenGL

class HouseControlView(QtGui.QWidget):
    def __init__(self, model, house):
        QtGui.QWidget.__init__(self)
        self.model = model
        self.house = house

        self.set_layout()
        self.connect_actions()

    def set_layout(self):
        controlLayout = QtGui.QGridLayout()
        controlLayout.setContentsMargins(0, 0, 0, 0)
        
        width_label = QtGui.QLabel('Width: ', self)
        height_label = QtGui.QLabel('Height: ', self)
        depth_label = QtGui.QLabel('Depth: ', self)
        rheight_label = QtGui.QLabel('Roof height: ', self)

        self.width_input = QtGui.QLineEdit('%.2f' % self.house.attr.width, self)
        self.height_input = QtGui.QLineEdit('%.2f' % self.house.attr.height, self)
        self.depth_input = QtGui.QLineEdit('%.2f' % self.house.attr.depth, self)
        self.rheight_input = QtGui.QLineEdit('%.2f' % self.house.attr.roof_height, self)

        controlLayout.addWidget(width_label, 0, 0)
        controlLayout.addWidget(self.width_input, 0, 1)
        controlLayout.addWidget(height_label, 0, 2)
        controlLayout.addWidget(self.height_input, 0, 3)
        controlLayout.addWidget(depth_label, 0, 4)
        controlLayout.addWidget(self.depth_input, 0, 5)
        controlLayout.addWidget(rheight_label, 0, 6)
        controlLayout.addWidget(self.rheight_input, 0, 7)

        fdoor_label = QtGui.QLabel('# front doors: ', self)
        sdoor_label = QtGui.QLabel('# side doors: ', self)
        bdoor_label = QtGui.QLabel('# back doors: ', self)

        self.fdoor_input = QtGui.QLineEdit('%d' % self.house.attr.num_front_doors, self)
        self.sdoor_input = QtGui.QLineEdit('%d' % self.house.attr.num_side_doors, self)
        self.bdoor_input = QtGui.QLineEdit('%d' % self.house.attr.num_back_doors, self)

        controlLayout.addWidget(fdoor_label, 0, 8)
        controlLayout.addWidget(self.fdoor_input, 0, 9)
        controlLayout.addWidget(sdoor_label, 0, 10)
        controlLayout.addWidget(self.sdoor_input, 0, 11)
        controlLayout.addWidget(bdoor_label, 1, 0)
        controlLayout.addWidget(self.bdoor_input, 1, 1)

        dwidth_label = QtGui.QLabel('Door width: ', self)
        dheight_label = QtGui.QLabel('Door height: ', self)

        self.dwidth_input = QtGui.QLineEdit ('%.2f' % self.house.attr.door_width, self)
        self.dheight_input = QtGui.QLineEdit('%.2f' % self.house.attr.door_height, self)

        controlLayout.addWidget(dwidth_label, 1, 2)
        controlLayout.addWidget(self.dwidth_input, 1, 3)
        controlLayout.addWidget(dheight_label, 1, 4)
        controlLayout.addWidget(self.dheight_input, 1, 5)

        fwin_label = QtGui.QLabel('# front wins: ', self)
        swin_label = QtGui.QLabel('# side wins: ', self)
        bwin_label = QtGui.QLabel('# back wins: ', self)

        self.fwin_input = QtGui.QLineEdit('%d' % self.house.attr.num_front_wins, self)
        self.swin_input = QtGui.QLineEdit('%d' % self.house.attr.num_side_wins, self)
        self.bwin_input = QtGui.QLineEdit('%d' % self.house.attr.num_back_wins, self)

        controlLayout.addWidget(fwin_label, 1, 6)
        controlLayout.addWidget(self.fwin_input, 1, 7)
        controlLayout.addWidget(swin_label, 1, 8)
        controlLayout.addWidget(self.swin_input, 1, 9)
        controlLayout.addWidget(bwin_label, 1, 10)
        controlLayout.addWidget(self.bwin_input, 1, 11)

        wwidth_label = QtGui.QLabel('Window width: ', self)
        wfbase_label = QtGui.QLabel('Base location: ', self)

        self.wwidth_input = QtGui.QLineEdit  ('%.2f' % self.house.attr.win_width, self)
        self.wfbase_input = QtGui.QLineEdit  ('%.2f' % self.house.attr.win_frame_base, self)

        controlLayout.addWidget(wwidth_label, 2, 0)
        controlLayout.addWidget(self.wwidth_input, 2, 1)
        controlLayout.addWidget(wfbase_label, 2, 2)
        controlLayout.addWidget(self.wfbase_input, 2, 3)

        self.update_btn = QtGui.QPushButton('Update', self)
        controlLayout.addWidget(self.update_btn, 2, 9, 1, 2)

        self.setLayout(controlLayout)

    def connect_actions(self):
        self.connect(self.update_btn, QtCore.SIGNAL('clicked()'), self.update_attr)

    def update_attr(self):
        new_width = self.width_input.text().toFloat()
        self.house.attr.width = new_width[0]

        new_height = self.height_input.text().toFloat()
        self.house.attr.height = new_height[0]

        new_depth = self.depth_input.text().toFloat()
        self.house.attr.depth = new_depth[0]

        new_rheight = self.rheight_input.text().toFloat()
        self.house.attr.roof_height = new_rheight[0]

        new_fdoor = self.fdoor_input.text().toFloat()
        self.house.attr.num_front_doors = int(new_fdoor[0])

        new_sdoor = self.sdoor_input.text().toFloat()
        self.house.attr.num_side_doors = int(new_sdoor[0])

        new_bdoor = self.bdoor_input.text().toFloat()
        self.house.attr.num_back_doors = int(new_bdoor[0])

        new_dwidth = self.dwidth_input.text().toFloat()
        self.house.attr.door_width = new_dwidth[0]

        new_dheight = self.dheight_input.text().toFloat()
        self.house.attr.door_height = new_dheight[0]

        new_fwin = self.fwin_input.text().toFloat()
        self.house.attr.num_front_wins = int(new_fwin[0])

        new_swin = self.swin_input.text().toFloat()
        self.house.attr.num_side_wins = int(new_swin[0])

        new_bwin = self.bwin_input.text().toFloat()
        self.house.attr.num_back_wins = int(new_bwin[0])

        new_wwidth = self.wwidth_input.text().toFloat()
        self.house.attr.win_width = new_wwidth[0]
        self.house.attr.win_frame_width = 0.5 * new_wwidth[0]
        self.house.attr.win_frame_height = 0.5 * new_wwidth[0]

        new_fbase = self.wfbase_input.text().toFloat()
        self.house.attr.win_frame_base = new_fbase[0]

        # compute model
        try:
            self.model.update_house(self.house)
        except:
            QtGui.QMessageBox.warning(self, 'Update House', 
                'Please reduce the number of doors/windows.')


class WallControlView(QtGui.QWidget):
    def __init__(self, model, wall):
        QtGui.QWidget.__init__(self)
        self.model = model
        self.wall = wall 

        self.set_layout()
        self.connect_actions()

    def set_layout(self):
        controlLayout = QtGui.QGridLayout()
        controlLayout.setContentsMargins(0, 0, 0, 0)
        
        width_label = QtGui.QLabel('Length: ', self)
        height_label = QtGui.QLabel('Height: ', self)
        depth_label = QtGui.QLabel('Depth: ', self)

        self.width_input = QtGui.QLineEdit('%.2f' % self.wall.attr.length, self)
        self.height_input = QtGui.QLineEdit('%.2f' % self.wall.attr.height, self)
        self.depth_input = QtGui.QLineEdit('%.2f' % self.wall.attr.depth, self)

        controlLayout.addWidget(width_label, 0, 0)
        controlLayout.addWidget(self.width_input, 0, 1)
        controlLayout.addWidget(height_label, 0, 2)
        controlLayout.addWidget(self.height_input, 0, 3)
        controlLayout.addWidget(depth_label, 0, 4)
        controlLayout.addWidget(self.depth_input, 0, 5)

        self.update_btn = QtGui.QPushButton('Update', self)
        controlLayout.addWidget(self.update_btn, 0, 7, 1, 2)

        self.setLayout(controlLayout)


    def connect_actions(self):
        self.connect(self.update_btn, QtCore.SIGNAL('clicked()'), self.update_attr)

    def update_attr(self):
        new_width = self.width_input.text().toFloat()
        self.wall.attr.length = new_width[0]

        new_height = self.height_input.text().toFloat()
        self.wall.attr.height = new_height[0]

        new_depth = self.depth_input.text().toFloat()
        self.wall.attr.depth = new_depth[0]

        self.model.update_wall(self.wall)

