#!/usr/bin/python

from OpenGL import GL
from PIL import Image
from math import *


N = 20
dt = 1.0 / N

class Pillar():
    def __init__(self, width, height):
        self.width = width
        self.height = height
        assert self.width >= 1, 'Pillar width must be at least 1'
        assert self.height >= 1, 'Pillar height must be at least 1'

        self.texture = GL.glGenTextures(1)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        im = Image.open('images/wall.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)

        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        self.object = GL.glGenLists(1)
        GL.glNewList(self.object, GL.GL_COMPILE)
        
        w = 0.5 * self.width
        dy = self.height / int(self.height)

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(self.height)):
            h = dt * (i % N)
            
            y = dy * i
            # front
            GL.glTexCoord2d(0, h, 0) 
            GL.glVertex3f(-w, y, w)
            GL.glTexCoord2d(dt, h, 0)
            GL.glVertex3f(w, y, w)
            GL.glTexCoord2d(dt, h+dt, 0)
            GL.glVertex3f(w, y+dy, w)
            GL.glTexCoord2d(0, h+dt, 0)
            GL.glVertex3f(-w, y+dy, w)

            # back
            GL.glTexCoord2d(0.5, h, 0)
            GL.glVertex3f(w, y, -w)
            GL.glTexCoord2d(0.75, h, 0)
            GL.glVertex3f(-w, y, -w)
            GL.glTexCoord2d(0.75, h+dt, 0)
            GL.glVertex3f(-w, y+dy, -w)
            GL.glTexCoord2d(0.5, h+dt, 0)
            GL.glVertex3f(w, y+dy, -w)

            # left
            GL.glTexCoord2d(0.75, h, 0)
            GL.glVertex3f(-w, y, -w)
            GL.glTexCoord2d(1, h, 0)    
            GL.glVertex3f(-w, y, w) 
            GL.glTexCoord2d(1, h+dt, 0)    
            GL.glVertex3f(-w, y+dy, w)
            GL.glTexCoord2d(0.75, h+dt, 0)
            GL.glVertex3f(-w, y+dy, -w)

            # right
            GL.glTexCoord2d(dt, h, 0)
            GL.glVertex3f(w, y, w)
            GL.glTexCoord2d(0.5, h, 0)    
            GL.glVertex3f(w, y, -w) 
            GL.glTexCoord2d(0.5, h+dt, 0)    
            GL.glVertex3f(w, y+dy, -w)
            GL.glTexCoord2d(dt, h+dt, 0)
            GL.glVertex3f(w, y+dy, w)

        GL.glEnd()
        GL.glEndList()

    def render(self):
        GL.glCallList(self.object)


class Wall():
    def __init__(self, width, height):
        self.width = width
        self.height = height
        assert self.width >= 1, 'Wall width must be at least 1'
        assert self.height >= 1, 'Wall height must be at least 1'

        self.texture = GL.glGenTextures(1)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        im = Image.open('images/wall.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)
        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        self.object = GL.glGenLists(1)
        GL.glNewList(self.object, GL.GL_COMPILE)

        w = 0.5 * self.width
        dx = self.width / int(self.width)
        dy = self.height / int(self.height)

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(self.width)):
            for j in xrange(int(self.height)):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dx * i
                y = dy * j
                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dy, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dy, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)

        GL.glEnd()
        GL.glEndList()

    def render(self):
        GL.glCallList(self.object)

class SideWall():
    def __init__(self, width, height, top, win_width, win_height):
        self.width = width
        self.height = height
        self.top = top
        self.win_width = win_width
        self.win_height = win_height
        assert self.width >= 1, 'Side wall width must be at least 1'
        assert self.height >= 1, 'Side wall height must be at least 1'
        assert self.top >= 1, 'Side wall roof height must be at least 1'
        assert self.win_width >= 1, 'Side wall window width must be at least 1'
        assert self.win_height >= 1, 'Side wall window height must be at least 1'

        self.texture = GL.glGenTextures(1)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        im = Image.open('images/wall.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)
        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        self.object = GL.glGenLists(1)
        GL.glNewList(self.object, GL.GL_COMPILE)

        w = 0.5 * self.width
        ww = 0.5 * self.win_width
        dx = self.width / int(self.width)
        dy = self.height / int(self.height)
        dtop = self.top / int(self.top)
        dwx = (w - ww) / int(w - ww)
        dwy = self.win_height / int(self.win_height)
        r = self.width / self.top
        ddx = 0.5 * dx
       
        GL.glDisable(GL.GL_TEXTURE_2D)
        GL.glBegin(GL.GL_QUADS)
        GL.glColor(0, 0, 0, 0)
        GL.glVertex3f(-ww, self.height, 0)
        GL.glVertex3f(ww, self.height, 0)
        GL.glVertex3f(ww, self.height - self.win_height, 0)
        GL.glVertex3f(-ww, self.height - self.win_height, 0)
        GL.glEnd()
        GL.glEnable(GL.GL_TEXTURE_2D)

 
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(self.width)):
            for j in xrange(int(self.height - self.win_height)):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dx * i
                y = dy * j
                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dy, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dy, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)
        GL.glEnd()

        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(w - ww)):
            for j in xrange(int(self.win_height)):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dwx * i
                y = dwy * j + self.height - self.win_height

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dy, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dy, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x+ww, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x+ww, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x+ww+dx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x+ww+dx, y+dy, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x+ww, y+dy, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x+ww+dx, y+dy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x+ww+dx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x+ww, y, 0)
        GL.glEnd()


        GL.glBegin(GL.GL_TRIANGLES)
        for j in xrange(int(self.top)):
            width = self.width - j * r * dx
            ww = 0.5 * width

            for i in xrange(int(width/dx)):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dx * i
                y = self.height + dtop * j

                GL.glTexCoord2d(h+0.125, v+dt, 0)
                GL.glVertex3f(x-ww+ddx, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dx, y, 0)

                GL.glTexCoord2d(h+0.125, v+dt, 0)
                GL.glVertex3f(x-ww+ddx, y+dy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww, y, 0)

            for i in xrange(int(width/dx)-1):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dx * i
                y = self.height + dtop * j

                GL.glTexCoord2d(h+0.125, v+dt, 0)
                GL.glVertex3f(x-ww+ddx, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww+dx, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dx+ddx, y+dy, 0)

                GL.glTexCoord2d(h+0.125, v+dt, 0)
                GL.glVertex3f(x-ww+ddx, y+dy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dx+ddx, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww+dx, y, 0)

        GL.glEnd()
        GL.glEndList()

    def render(self):
        GL.glCallList(self.object)


class Door():
    def __init__(self, width, height, total_height):
        self.width = width
        self.height = height
        self.total_height = total_height
        assert self.width >= 1, 'Door width must be at least 1'
        assert self.height >= 1, 'Door height must be at least 1'
        assert self.total_height - self.height >= 1, 'Fasade height must be at lest 1 unit higher than door height'

        self.door_tex = GL.glGenTextures(1)
        im = Image.open('images/door.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.door_tex)
        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        self.wall_tex = GL.glGenTextures(1)
        im = Image.open('images/wall.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.wall_tex)
        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        self.door = GL.glGenLists(1)
        self.wall = GL.glGenLists(1)

        w = 0.5 * self.width
        dx = self.width / int(self.width)
        dy = self.height / int(self.height)
        dny = (self.total_height - self.height) / int(self.total_height - self.height)

        GL.glNewList(self.door, GL.GL_COMPILE)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.door_tex)
        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(self.width)):
            for j in xrange(int(self.height)):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dx * i
                y = dy * j
                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dy, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dy, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)

        GL.glEnd()
        GL.glEndList()

        GL.glNewList(self.wall, GL.GL_COMPILE)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.wall_tex)
        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(self.width)):
            for j in xrange(int(self.total_height - self.height)):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dx * i
                y = self.height + dy * j

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dny, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dny, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-w, y+dny, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-w+dx, y+dny, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-w+dx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-w, y, 0)

        GL.glEnd()
        GL.glEndList()



    def render(self):
        GL.glCallList(self.wall)
        GL.glCallList(self.door)


class Window():
    def __init__(self, width, height, base, wall_width, wall_height):
        self.width = width
        self.height = height
        self.base = base
        self.wall_width = wall_width
        self.wall_height = wall_height
        assert self.width >= 1, 'Window width must be at least 1'
        assert self.height >= 1, 'Window height must be at least 1'

        self.texture = GL.glGenTextures(1)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        im = Image.open('images/wall.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)
        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        self.object = GL.glGenLists(1)
        GL.glNewList(self.object, GL.GL_COMPILE)

        ww = 0.5 * self.wall_width
        w = 0.5 * self.width
        u = self.height + self.base

        dx = self.wall_width / int(self.wall_width)
        dy = self.wall_height / int(self.wall_height)
        db = self.base / int(self.base)
        dwx = (self.wall_width - self.width) / int(self.wall_width - self.width)
        dwy = self.height / int(self.height)

        GL.glDisable(GL.GL_TEXTURE_2D)
        GL.glBegin(GL.GL_QUADS)
        GL.glColor(0, 0, 0, 0)
        GL.glVertex3f(-w, self.base, 0)
        GL.glVertex3f(w, self.base, 0)
        GL.glVertex3f(w, u, 0)
        GL.glVertex3f(-w, u, 0)
        GL.glEnd()
        GL.glEnable(GL.GL_TEXTURE_2D)

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(self.wall_width)):
            for j in xrange(int(self.base)):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dx * i
                y = db * j

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-ww, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-ww+dx, y+dy, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-ww, y+dy, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-ww+dx, y+dy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww, y, 0)

        for i in xrange(int(self.wall_width)):
            for j in range(int(u), int(self.wall_height)):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dx * i
                y = dy * j

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-ww, y+dy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-ww+dx, y+dy, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-ww, y+dy, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-ww+dx, y+dy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww, y, 0)


        for i in xrange(int(0.5 * (self.wall_width - self.width))):
            for j in xrange(int(self.height)):
                h = dt * (i % N)
                v = dt * (j % N)

                x = dwx * i
                y = self.base + dwy * j

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-ww, y+dwy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dwx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-ww+dwx, y+dwy, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x-ww, y+dwy, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x-ww+dwx, y+dwy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x-ww+dwx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x-ww, y, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x+w, y+dwy, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x+w, y, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x+w+dwx, y, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x+w+dwx, y+dwy, 0)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(x+w, y+dwy, 0)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(x+w+dwx, y+dwy, 0)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(x+w+dwx, y, 0)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(x+w, y, 0)

        GL.glEnd()
        GL.glEndList()


    def render(self):
        GL.glCallList(self.object)


class RecRoof():
    def __init__(self, width, height, depth):
        self.width = width
        self.height = height
        self.depth = depth

        self.texture = GL.glGenTextures(2)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture[0])
        im = Image.open('images/roof.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)
        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture[1])
        im = Image.open('images/roof2.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)
        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)


        self.object = GL.glGenLists(1)
        GL.glNewList(self.object, GL.GL_COMPILE)

        w = 0.5 * self.width
        d = 0.5 * self.depth
        theta = atan(self.height/d)
        sint = sin(theta)
        cost = cos(theta)
        l = sqrt(self.height * self.height + d * d) + 1

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture[0])
        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(self.width)):
            for j in xrange(int(l)):
                h = dt * (i % N)
                v = dt * (j % N)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(i-w, self.height - (j + 1) * sint, (j + 1) * cost)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(i-w, self.height - j * sint, j * cost)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(i-w+1, self.height - j * sint, j * cost)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(i-w+1, self.height - (j + 1) * sint, (j + 1) * cost)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(i-w, self.height - (j + 1) * sint, (j + 1) * cost)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(i-w+1, self.height - (j + 1) * sint, (j + 1) * cost)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(i-w+1, self.height - j * sint, j * cost)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(i-w, self.height - j * sint, j * cost)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(i-w, self.height - (j + 1) * sint, -(j + 1) * cost)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(i-w, self.height - j * sint, -j * cost)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(i-w+1, self.height - j * sint, -j * cost)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(i-w+1, self.height - (j + 1) * sint, -(j + 1) * cost)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(i-w, self.height - (j + 1) * sint, -(j + 1) * cost)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(i-w+1, self.height - (j + 1) * sint, -(j + 1) * cost)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(i-w+1, self.height - j * sint, -j * cost)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(i-w, self.height - j * sint, -j * cost)
        GL.glEnd()

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture[1])
        GL.glBegin(GL.GL_TRIANGLES)

        GL.glTexCoord2d(0.5, 1, 0)
        GL.glVertex3f(-w, self.height, 0)
        GL.glTexCoord2d(0, 0, 0)
        GL.glVertex3f(-w, 0, -d)
        GL.glTexCoord2d(1, 0, 0)
        GL.glVertex3f(-w, 0, d)

        GL.glTexCoord2d(0.5, 1, 0)
        GL.glVertex3f(-w, self.height, 0)
        GL.glTexCoord2d(1, 0, 0)
        GL.glVertex3f(-w, 0, d)
        GL.glTexCoord2d(0, 0, 0)
        GL.glVertex3f(-w, 0, -d)

        GL.glTexCoord2d(0.5, 1, 0)
        GL.glVertex3f(w, self.height, 0)
        GL.glTexCoord2d(0, 0, 0)
        GL.glVertex3f(w, 0, -d)
        GL.glTexCoord2d(1, 0, 0)
        GL.glVertex3f(w, 0, d)

        GL.glTexCoord2d(0.5, 1, 0)
        GL.glVertex3f(w, self.height, 0)
        GL.glTexCoord2d(1, 0, 0)
        GL.glVertex3f(w, 0, d)
        GL.glTexCoord2d(0, 0, 0)
        GL.glVertex3f(w, 0, -d)
        GL.glEnd()
        GL.glEndList()

    def render(self):
        GL.glCallList(self.object)

class SharpRoof():
    def __init__(self, width, height, depth):
        self.width = width
        self.height = height
        self.depth = depth

        self.texture = GL.glGenTextures(1)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        im = Image.open('images/roof.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)
        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        self.object = GL.glGenLists(1)
        GL.glNewList(self.object, GL.GL_COMPILE)

        w = 0.5 * self.width
        d = 0.5 * self.depth
        theta = atan(self.height/d)
        sint = sin(theta)
        cost = cos(theta)
        l = sqrt(self.height * self.height + d * d) + 1
        
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.texture)
        GL.glBegin(GL.GL_QUADS)

        for i in xrange(int(self.width)):
            for j in xrange(int(l)):
                h = dt * (i % N)
                v = dt * (j % N)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(i-w, self.height - (j + 1) * sint, (j + 1) * cost)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(i-w, self.height - j * sint, j * cost)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(i-w+1, self.height - j * sint, j * cost)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(i-w+1, self.height - (j + 1) * sint, (j + 1) * cost)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(i-w, self.height - (j + 1) * sint, (j + 1) * cost)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(i-w+1, self.height - (j + 1) * sint, (j + 1) * cost)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(i-w+1, self.height - j * sint, j * cost)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(i-w, self.height - j * sint, j * cost)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(i-w, self.height - (j + 1) * sint, -(j + 1) * cost)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(i-w, self.height - j * sint, -j * cost)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(i-w+1, self.height - j * sint, -j * cost)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(i-w+1, self.height - (j + 1) * sint, -(j + 1) * cost)

                GL.glTexCoord2d(h, v+dt, 0)
                GL.glVertex3f(i-w, self.height - (j + 1) * sint, -(j + 1) * cost)
                GL.glTexCoord2d(h+dt, v+dt, 0)
                GL.glVertex3f(i-w+1, self.height - (j + 1) * sint, -(j + 1) * cost)
                GL.glTexCoord2d(h+dt, v, 0)
                GL.glVertex3f(i-w+1, self.height - j * sint, -j * cost)
                GL.glTexCoord2d(h, v, 0)
                GL.glVertex3f(i-w, self.height - j * sint, -j * cost)

        GL.glEnd()
        GL.glEndList()

    def render(self):
        GL.glCallList(self.object)


