#!/usr/bin/python

from math import *
from OpenGL import GL
from PIL import Image

class Ground():
    def __init__(self, width, length):
        self.width = width
        self.length = length

        self.grass = GL.glGenTextures(1)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.grass)
        im = Image.open('images/ground.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)

        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        self.stairs = GL.glGenTextures(1)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.stairs)
        im = Image.open('images/stairs.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)

        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)

        self.stone = GL.glGenTextures(1)
        GL.glBindTexture(GL.GL_TEXTURE_2D, self.stone)
        im = Image.open('images/terrace.jpg')
        try:
            data = im.tostring("raw", "RGBA", 0, -1)
        except SystemError:
            data = im.tostring("raw", "RGBX", 0, -1)
        GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)

        GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                   0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)


        self.height_map = Image.new("L", [width+1, length+1])
        data = list(self.height_map.getdata())
        self.height_map.putdata(data, 1, 255)

        self.stair_map = Image.new("L", [width+1, length+1])

        self.object = 0
        self.gen_list()

    def gen_list(self):
        if self.object != 0:
            GL.glDeleteLists(self.object, 1)

        self.object = GL.glGenLists(1)
        GL.glNewList(self.object, GL.GL_COMPILE)
        w = 0.5 * self.width
        l = 0.5 * self.length
        self.d = 10.0;
        dx = self.d * self.width / int(self.width)
        dy = self.d * self.length / int(self.length)

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.grass)
        GL.glBegin(GL.GL_QUADS)
        
        for i in xrange(int(self.width / self.d)):
            for j in xrange(int(self.length / self.d)):
                x = dx * i
                y = dy * j

                h1 = self.height_map.getpixel((x, y))
                h2 = self.height_map.getpixel((x+dx, y))
                h3 = self.height_map.getpixel((x, y+dy))
                h4 = self.height_map.getpixel((x+dx, y+dy))

                if ((self.stair_map.getpixel((x, y)) == 0 or
                     self.stair_map.getpixel((x+dx, y)) == 0 or
                     self.stair_map.getpixel((x, y+dy)) == 0 or
                     self.stair_map.getpixel((x+dx, y+dy)) == 0) and
                    (h1 == h2 and h1 == h3 and h3 == h4)):

                    GL.glTexCoord2d(x/self.width, (y + dy)/self.length, 0)
                    GL.glVertex3f(x-w, self.height_map.getpixel((x, y+dy)), y-l+dy)

                    GL.glTexCoord2d(x/self.width, y/self.length, 0)
                    GL.glVertex3f(x-w+dx, self.height_map.getpixel((x+dx, y+dy)), y-l+dy)

                    GL.glTexCoord2d((x + dx)/self.width, y/self.length, 0)
                    GL.glVertex3f(x-w+dx, self.height_map.getpixel((x+dx, y)), y-l)

                    GL.glTexCoord2d((x + dx)/self.width, (y + dy)/self.length, 0)
                    GL.glVertex3f(x-w, self.height_map.getpixel((x, y)), y-l)
        GL.glEnd()

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.stairs)
        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(self.width / self.d)):
            for j in xrange(int(self.length / self.d)):
                x = dx * i
                y = dy * j

                if (self.stair_map.getpixel((x, y)) == 1 and
                    self.stair_map.getpixel((x+dx, y)) == 1 and
                    self.stair_map.getpixel((x, y+dy)) == 1 and
                    self.stair_map.getpixel((x+dx, y+dy)) == 1):

                    GL.glTexCoord2d(0, 0, 0)
                    GL.glVertex3f(x-w, self.height_map.getpixel((x, y+dy)), y-l+dy)

                    GL.glTexCoord2d(1, 0, 0)
                    GL.glVertex3f(x-w+dx, self.height_map.getpixel((x+dx, y+dy)), y-l+dy)

                    GL.glTexCoord2d(1, 1, 0)
                    GL.glVertex3f(x-w+dx, self.height_map.getpixel((x+dx, y)), y-l)

                    GL.glTexCoord2d(0, 1, 0)
                    GL.glVertex3f(x-w, self.height_map.getpixel((x, y)), y-l)
        GL.glEnd()

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.stone)
        GL.glBegin(GL.GL_QUADS)
        for i in xrange(int(self.width / self.d)):
            for j in xrange(int(self.length / self.d)):
                x = dx * i
                y = dy * j

                h1 = self.height_map.getpixel((x, y))
                h2 = self.height_map.getpixel((x+dx, y))
                h3 = self.height_map.getpixel((x, y+dy))
                h4 = self.height_map.getpixel((x+dx, y+dy))

                if (h1 != h2 or h1 != h3 or h1 != h4):

                    GL.glTexCoord2d(x/self.width, (y + dy)/self.length, 0)
                    GL.glVertex3f(x-w, self.height_map.getpixel((x, y+dy)), y-l+dy)

                    GL.glTexCoord2d(x/self.width, y/self.length, 0)
                    GL.glVertex3f(x-w+dx, self.height_map.getpixel((x+dx, y+dy)), y-l+dy)

                    GL.glTexCoord2d((x + dx)/self.width, y/self.length, 0)
                    GL.glVertex3f(x-w+dx, self.height_map.getpixel((x+dx, y)), y-l)

                    GL.glTexCoord2d((x + dx)/self.width, (y + dy)/self.length, 0)
                    GL.glVertex3f(x-w, self.height_map.getpixel((x, y)), y-l)
        GL.glEnd()


        GL.glEndList()

    def elevation(self, angle, start, end):
        upnear = [0, 0]

        x = end[0] - start[0]
        y = end[1] - start[1]

        upfar = [x * cos(angle), x * sin(angle)]
        bonear = [-y * sin(angle), y * cos(angle)]
        bofar = [end[0] - start[0], end[1] - start[1]]
        bofar = [x * cos(angle) - y * sin(angle), x * sin(angle) + y * cos(angle)]

        k1 = tan(angle)
        k2 = tan(angle - 0.5 * pi)
        b1 = 0
        b2 = 0
        b3 = bonear[1] - k1 * bonear[0]
        b4 = upfar[1] - k2 * upfar[0]

        self.k = tan(0.5 * pi + angle)
        bb1 = start[1] - self.k * start[0]
        bb2 = start[1] + bofar[1] - self.k * (start[0] + bofar[0]) 
        self.small_b = min(bb1, bb2)
        self.big_b = max(bb1, bb2)

        xmin = min(min(upnear[0], upfar[0]), min(bonear[0], bofar[0]))
        xmax = max(max(upnear[0], upfar[0]), max(bonear[0], bofar[0]))
        ymin = min(min(upnear[1], upfar[1]), min(bonear[1], bofar[1]))
        ymax = max(max(upnear[1], upfar[1]), max(bonear[1], bofar[1]))

        init_h = self.height_map.getpixel((start[0], start[1]))
        for i in range(int(xmin), int(xmax)):
            for j in range(int(ymin), int(ymax + 1.5)):
                if (k1 * i + b1 > j or k1 * i + b3 < j or
                    k2 * i + b2 < j or k2 * i + b4 > j) :
                        continue

                x = int(start[0] + i)
                if x < 0 : x = 0
                if x > self.width: x = self.width

                y = int(start[1] + j)
                if y < 0 : y = 0
                if y > self.length: y = self.length

                self.height_map.putpixel((x, y), init_h - 0.3 * j)
                self.stair_map.putpixel((x, y), 1)

        d = 255 - 0.3 * int(ymax - 0.5)
        if angle < 0:
            y = k2 * int(xmax + 0.5) + b4
            xstart = (y - b2) / k2
            ystart = k2 * xstart + b2
            xstart = xstart + start[0]
            ystart = ystart + start[1]

            for i in range(int(xstart + 0.5), self.width):
                for j in range(int(ystart - 0.5), self.length+1):
                    x = i - start[0]
                    y = j - start[1]
                    if (k2 * x + b2 < y or k2 * x + b4 > y) : continue
                    self.height_map.putpixel((i, j), d)

        elif angle > 0:
            y = k2 * int(xmin + 0.5) + b4
            xend = (y - b2) / k2
            ystart = k2 * xend + b2
            xend = xend + start[0]
            ystart = ystart + start[1]
            if xend > self.width: xend = self.width

            for i in range(0, int(xend + 1.5)):
                for j in range(int(ystart - 0.5), self.length+1):
                    x = i - start[0]
                    y = j - start[1]
                    if (k2 * x + b2 < y or k2 * x + b4 > y) : continue
                    self.height_map.putpixel((i, j), d)

        self.gen_list()
        
    def terrace(self, lines):
        for l in xrange(len(lines)-1):
            l1 = lines[l].line()
            k1 = (l1.y2() - l1.y1()) / (l1.x2() - l1.x1())
            b1 = l1.y1() - k1 * l1.x1()

            l2 = lines[l+1].line()
            k2 = (l2.y2() - l2.y1()) / (l2.x2() - l2.x1())
            b2 = l2.y1() - k2 * l2.x1()

            h = self.height_map.getpixel((l1.x1(), l1.y1()))

            x1 = (l1.y1() - self.small_b) / self.k
            x2 = (l2.y1() - self.small_b) / self.k
            if l1.x1() < l1.x2():
                xmin1 = min(x1, x2)
                xmin2 = min(l1.x1(), l2.x1())
                rangex = range(int(min(xmin1, xmin2)), self.width + 1)
                y1end = k1 * self.width + b1
                if y1end > self.length: y1end = self.length
                y2end = k2 * self.width + b2
                if y2end > self.length: y2end = self.length
            else:
                xmax1 = max(x1, x2)
                xmax2 = max(l1.x1(), l2.x1())
                rangex = range(0, int(max(xmax1, xmax2)))
                y1end = b1
                if y1end > self.length: y1end = self.length
                y2end = b2
                if y2end > self.length: y2end = self.length

            ymin = min(l1.y1(), y1end)
            ymax = max(l2.y1(), y2end)
            # area below line 1, above l2, and right to elevation line are marked as h
            for i in rangex:
                for j in range(int(ymin-0.5), int(ymax+1.5)):
                    yy1 = self.k * i + self.small_b
                    if self.k > 0:
                        if l1.x1() > l1.x2() and j < yy1: continue
                        if l1.x1() < l1.x2() and j > yy1: continue
                    if self.k < 0:
                        if l1.x1() > l1.x2() and j > yy1: continue
                        if l1.x1() < l1.x2() and j < yy1: continue

                    y1 = k1 * i + b1
                    y2 = k2 * i + b2
                    if j >= y1 and j <= y2: # below l1 and above l2 
                        if i < 0 or i > self.width or j < 0 or j > self.length:
                            continue
                        if self.height_map.getpixel((i, j)) > h:
                            self.height_map.putpixel((i, j), h)

        self.gen_list()

    def get_height(self, x, y):
        x = int(x)
        y = int(y)
        return self.height_map.getpixel((x, y))

    def render(self):
        GL.glCallList(self.object)
       
