#!/usr/bin/python

import sys
import math
from PyQt4 import QtCore, QtGui, QtOpenGL
from OpenGL import GL, GLU

from utils import *
from attributes import *
from attributes_controls import *
from shape_grammar import *

HOUSE = 1
INCAWALL = 2
TERRACE = 3
HA = HouseAttributes()
WA = WallAttributes()

class GLWidget(QtOpenGL.QGLWidget):
    def __init__(self, parent=None):
        QtOpenGL.QGLWidget.__init__(self, parent)
        self.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.pos = QtCore.QPoint()
        self.trans = [0, -400, -1000]
        self.rot = [10, 0, 0]
        self.s = 1.0

    def initializeGL(self):
        self.qglClearColor(QtGui.QColor(0, 0, 0))

        GL.glEnable(GL.GL_TEXTURE_2D)
        GL.glShadeModel(GL.GL_SMOOTH)
        GL.glClearColor(0.0, 0.0, 0.0, 0.0)
        GL.glClearDepth(1.0)
        GL.glEnable(GL.GL_DEPTH_TEST)
        GL.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST)

        # 0: negz, 1: posz, 2: negx, 3: posx, 4: negy, 5: posy
        self.skybox = GL.glGenTextures(6)
        for i in xrange(6):
            GL.glBindTexture(GL.GL_TEXTURE_2D, self.skybox[i])
            im = Image.open('images/hills' + str(i) + '.jpg')
            try:
                data = im.tostring("raw", "RGBA", 0, -1)
            except SystemError:
                data = im.tostring("raw", "RGBX", 0, -1)
            GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)

            GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST)
	    GL.glTexParameterf(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST)
	    GL.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL)
            GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, im.size[0], im.size[1],
                       0, GL.GL_RGBA, GL.GL_UNSIGNED_BYTE, data)


    def paintGL(self):
        GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
        GL.glClearColor(0.1, 0.2, 0.3, 0.0)

        GL.glMatrixMode(GL.GL_PROJECTION)
        GL.glLoadIdentity()
        GLU.gluPerspective(40.0, 1, 0.1, 5000.0)

        GL.glMatrixMode(GL.GL_MODELVIEW)

        GL.glLoadIdentity()
        GL.glRotate(self.rot[0]+10, 1, 0, 0)
        GL.glRotate(self.rot[1], 0, 1, 0)
        GL.glRotate(self.rot[2]-10, 0, 0, 1)

        GL.glPushAttrib(GL.GL_ENABLE_BIT)
        GL.glEnable(GL.GL_TEXTURE_2D)
        GL.glDisable(GL.GL_DEPTH_TEST)
        GL.glDisable(GL.GL_LIGHTING)
        GL.glDisable(GL.GL_BLEND)
        GL.glColor4f(1, 1, 1, 1)

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.skybox[0])
        GL.glBegin(GL.GL_QUADS)
        GL.glTexCoord2f(0, 0)
        GL.glVertex3f(2000, -2000, -2000)
        GL.glTexCoord2f(1, 0)
        GL.glVertex3f(-2000, -2000, -2000)
        GL.glTexCoord2f(1, 1)
        GL.glVertex3f(-2000, 2000, -2000)
        GL.glTexCoord2f(0, 1)
        GL.glVertex3f(2000, 2000, -2000)
        GL.glEnd()

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.skybox[1])
        GL.glBegin(GL.GL_QUADS)
        GL.glTexCoord2f(0, 0)
        GL.glVertex3f(-2000, -2000, 2000)
        GL.glTexCoord2f(1, 0)
        GL.glVertex3f(2000, -2000, 2000)
        GL.glTexCoord2f(1, 1)
        GL.glVertex3f(2000, 2000, 2000)
        GL.glTexCoord2f(0, 1)
        GL.glVertex3f(-2000, 2000, 2000)
        GL.glEnd()

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.skybox[2])
        GL.glBegin(GL.GL_QUADS)
        GL.glTexCoord2f(0, 0)
        GL.glVertex3f(-2000, -2000, -2000)
        GL.glTexCoord2f(1, 0)
        GL.glVertex3f(-2000, -2000, 2000)
        GL.glTexCoord2f(1, 1)
        GL.glVertex3f(-2000, 2000, 2000)
        GL.glTexCoord2f(0, 1)
        GL.glVertex3f(-2000, 2000, -2000)
        GL.glEnd()

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.skybox[3])
        GL.glBegin(GL.GL_QUADS)
        GL.glTexCoord2f(0, 0)
        GL.glVertex3f(2000, -2000, 2000)
        GL.glTexCoord2f(1, 0)
        GL.glVertex3f(2000, -2000, -2000)
        GL.glTexCoord2f(1, 1)
        GL.glVertex3f(2000, 2000, -2000)
        GL.glTexCoord2f(0, 1)
        GL.glVertex3f(2000, 2000, 2000)
        GL.glEnd()

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.skybox[4])
        GL.glBegin(GL.GL_QUADS)
        GL.glTexCoord2f(0, 0)
        GL.glVertex3f(-2000, -2000, -2000)
        GL.glTexCoord2f(1, 0)
        GL.glVertex3f(-2000, -2000, 2000)
        GL.glTexCoord2f(1, 1)
        GL.glVertex3f(2000, -2000, 2000)
        GL.glTexCoord2f(0, 1)
        GL.glVertex3f(2000, -2000, -2000)
        GL.glEnd()

        GL.glBindTexture(GL.GL_TEXTURE_2D, self.skybox[5])
        GL.glBegin(GL.GL_QUADS)
        GL.glTexCoord2f(0, 0)
        GL.glVertex3f(-2000, 2000, -2000)
        GL.glTexCoord2f(1, 0)
        GL.glVertex3f(-2000, 2000, 2000)
        GL.glTexCoord2f(1, 1)
        GL.glVertex3f(2000, 2000, 2000)
        GL.glTexCoord2f(0, 1)
        GL.glVertex3f(2000, 2000, -2000)
        GL.glEnd()

        GL.glPopAttrib()

        GL.glPushMatrix()
        GL.glLoadIdentity()
        GL.glTranslate(self.trans[0], self.trans[1], self.trans[2])
        GL.glScale(self.s, self.s, self.s)
        GL.glRotate(self.rot[0], 1, 0, 0)
        GL.glRotate(self.rot[1], 0, 1, 0)
        GL.glRotate(self.rot[2], 0, 0, 1)
        if self.model:
            self.model.render()
        GL.glPopMatrix()

    def wheelEvent(self, event):
        self.s += event.delta() / 3600.0
        if (self.s < 0.1): self.s = 0.1
        self.updateGL()

    def mousePressEvent(self, event):
        self.pos = QtCore.QPoint(event.pos())

    def mouseMoveEvent(self, event):
        dx = event.x() - self.pos.x()
        dy = event.y() - self.pos.y()
        self.pos = QtCore.QPoint(event.pos())

        if event.buttons() & QtCore.Qt.LeftButton:
            self.trans[0] += 0.5 * dx
            self.trans[1] -= 0.5 * dy

        elif event.buttons() & QtCore.Qt.MidButton:
            self.rot[0] += 0.1 * dy
            self.rot[1] += 0.1 * dx

        self.updateGL()

    def set_model(self, model):
        self.model = model 
        self.model.register_view(self)

    def update(self):
        self.updateGL()

class PlanView(QtGui.QGraphicsView):
    def __init__(self, parent=None):
        QtGui.QGraphicsView.__init__(self, parent)

        self.app = parent

        self.scene = QtGui.QGraphicsScene(self)
        self.scene.setSceneRect(-200, -200, 200, 200)
        self.setScene(self.scene)

        self.house_pen = QtGui.QPen()
        self.house_pen.setColor(QtCore.Qt.red)
        self.house_pen.setWidth(3)

        self.wall_pen = QtGui.QPen()
        self.wall_pen.setColor(QtCore.Qt.green)
        self.wall_pen.setWidth(3)

        self.terrace_pen = QtGui.QPen()
        self.terrace_pen.setColor(QtCore.Qt.blue)
        self.terrace_pen.setWidth(3)

        self.mouse_pressed = 0
        self.lines = []
        self.selected_lines = []
        self.temp_line = 0

        self.pos = QtCore.QPoint()

    def mousePressEvent(self, event):
        if self.mouse_pressed == 0:
            self.mouse_pressed = 1
            self.pos = self.mapToScene(event.pos())
            if self.app.mode == HOUSE:
                pen = self.house_pen
            elif self.app.mode == INCAWALL:
                pen = self.wall_pen
            elif self.app.mode == TERRACE:
                pen = self.terrace_pen
            self.cur_line = self.scene.addLine(QtCore.QLineF(QtCore.QPointF(self.pos.x(), \
                self.pos.y()), QtCore.QPointF(self.pos.x(), self.pos.y())), pen)

    def mouseReleaseEvent(self, event):
        self.mouse_pressed = 0
        scene_pos = self.mapToScene(event.pos())
        self.selected_lines.append(self.cur_line)

    def mouseMoveEvent(self, event):
        if self.mouse_pressed == 1:
            scene_pos = self.mapToScene(event.pos())
            self.cur_line.setLine(QtCore.QLineF(QtCore.QPointF(self.pos.x(), \
                self.pos.y()), QtCore.QPointF(scene_pos.x(), scene_pos.y())))
            self.update()


    def keyPressEvent(self, event):
        if event.text() == 'c':
            self.clear_selection()

        if event.text() == 'r':
            self.reset()
            
        elif event.text() == 'a':
            if self.app.mode == HOUSE:
                if len(self.selected_lines) != 4:
                    QtGui.QMessageBox.warning(self, 'Create New House', 
                        'Select four lines to create a house!')
                else:
                    attr = HA.generate_attributes(self.selected_lines)
                    h = House(attr)
                    h.set_translation(comp_house_trans(self.selected_lines, 
                        self.size.width(), self.size.height(), self.app.model))
                    h.set_rotation(comp_house_rot(self.selected_lines))
                    self.app.model.add_house(h)
                    self.add_lines()

            elif self.app.mode == INCAWALL:
                if len(self.selected_lines) != 1:
                    QtGui.QMessageBox.warning(self, 'Create New Wall',
                        'Select one line to create a wall!')
                else:
                    attr = WA.generate_attributes(self.selected_lines)
                    w = IncaWall(attr)
                    w.set_translation(comp_wall_trans(self.selected_lines,
                        self.size, self.app.model))
                    w.set_rotation(comp_wall_rot(self.selected_lines))
                    self.app.model.add_wall(w)
                    self.add_lines()

            self.clear_selection()

        elif event.text() == 's':
            if self.app.mode == TERRACE:
                if len(self.selected_lines) != 2:
                    QtGui.QMessageBox.warning(self, 'Create stairs',
                        'Select two lines (length and width) to create stairs!')
                else:
                    [angle, start, end] = comp_stairs(self.selected_lines)
                    self.app.model.elevation(angle, start, end)

                    self.add_lines()
                self.clear_selection()

        elif event.text() == 't':
            if self.app.mode == TERRACE:
                if len(self.selected_lines) < 2:
                    QtGui.QMessageBox.warning(self, 'Create terraces', 
                        'Select at least two lines to indicate the locations of the terraces!')
                else:
                    self.selected_lines.append(self.scene.addLine(0, self.size.height()-1, \
                                    self.size.width(), self.size.height()-1))
                    self.app.model.terrace(self.selected_lines)
                    self.add_lines()
                self.clear_selection()
    
    def add_lines(self):
        for line in self.selected_lines:
            self.lines.append(line)
        self.selected_lines = []

    def clear_selection(self):
        for line in self.selected_lines:
            self.scene.removeItem(line)
        self.selected_lines = []

    def reset(self):
        self.clear_selection()
        for line in self.lines:
            self.scene.removeItem(line)
        self.lines = []
        self.app.model.clear()
        self.app.model.update()

        
    def load_image(self, image, size):
        self.reset()
        self.size = size
        self.scene.setBackgroundBrush(QtGui.QBrush(QtCore.Qt.white, image.pixmap()))
        self.setSceneRect(0, 0, size.width(), size.height())
        self.app.model.ground_size(size.width(), size.height())
            

class MainWindow(QtGui.QMainWindow):
    def __init__(self):
        QtGui.QMainWindow.__init__(self)
        self.resize(980, 740)
        self.setWindowTitle('Inca Reconstruction')
    
        self.set_menu()
        self.set_layout()
        self.set_manual()

        self.model = IncaModel()
        self.model.register_view(self)
        self.plan_image = 0
        self.mode = HOUSE
        self.render_view.set_model(self.model)

        self.statusBar().showMessage('House Mode')

    def set_menu(self):
        open = QtGui.QAction('Open', self)
        open.setShortcut('Ctrl+O')
        self.connect(open, QtCore.SIGNAL('triggered()'), self.slotFile)

        exit = QtGui.QAction('Exit', self)
        exit.setShortcut('Ctrl+Q')
        self.connect(exit, QtCore.SIGNAL('triggered()'), QtCore.SLOT('close()'))

        self.hmode = QtGui.QAction('House', self)
        self.hmode.setCheckable(True)
        self.hmode.setShortcut('Ctrl+H')
        self.connect(self.hmode, QtCore.SIGNAL('triggered()'), self.slotHouseMode)

        self.wmode = QtGui.QAction('IncaWall', self)
        self.wmode.setCheckable(True)
        self.wmode.setShortcut('Ctrl+W')
        self.connect(self.wmode, QtCore.SIGNAL('triggered()'), self.slotWallMode)

        self.tmode = QtGui.QAction('Terrace', self)
        self.tmode.setCheckable(True)
        self.tmode.setShortcut('Ctrl+T')
        self.connect(self.tmode, QtCore.SIGNAL('triggered()'), self.slotTerraceMode)

        self.random_gen = QtGui.QAction('Generate', self)
        self.random_gen.setShortcut('Ctrl+R')
        self.connect(self.random_gen, QtCore.SIGNAL('triggered()'), self.slotGenerate)

        self.manual = QtGui.QAction('User Manual', self)
        self.manual.setShortcut('Ctrl+M')
        self.connect(self.manual, QtCore.SIGNAL('triggered()'), self.slotManual)

        menubar = self.menuBar()
        file = menubar.addMenu('&File')
        file.addAction(open)
        file.addAction(exit)

        mode = menubar.addMenu('&Mode')
        mode.addAction(self.hmode)
        mode.addAction(self.wmode)
        mode.addAction(self.tmode)

        random = menubar.addMenu('&Random')
        random.addAction(self.random_gen)

        help = menubar.addMenu('&Help')
        help.addAction(self.manual)

    def set_layout(self):
        self.mainLayout = QtGui.QGridLayout()
        self.setCentralWidget(QtGui.QWidget());
        self.centralWidget().setLayout(self.mainLayout)

        self.scene_view = PlanView(self)
        self.scene_view.setRenderHint(QtGui.QPainter.Antialiasing)
        self.scene_view.setMinimumSize(490, 500)
        self.scene_view.setMaximumSize(490, 500)
        self.mainLayout.addWidget(self.scene_view, 0, 0)

        self.render_view = GLWidget(self)
        self.render_view.setMinimumSize(490, 500)
        self.render_view.setMaximumSize(490, 500)
        self.mainLayout.addWidget(self.render_view, 0, 1)

        self.tab = QtGui.QTabWidget(self)
        self.tab.setMinimumSize(980, 150)
        self.tab.setMaximumSize(980, 150)
        self.mainLayout.addWidget(self.tab, 1, 0, 1, 2)


    def set_manual(self):
        self.manual_box = QtGui.QMessageBox(self)
        self.manual_box.setWindowTitle('User Manual for Inca Recon')
        self.manual_box.setTextFormat(QtCore.Qt.RichText)

        help_text = 'The application contains the following shortcuts: '
        help_text += '<ul><li>Ctrl+Q : Quit the application.<li>'
        help_text += '<li>Ctrl+O : Open a plan image. <br>'
        help_text += 'The plan image is displayed in the Plan View widget.</li>'
        help_text += '<li>Ctrl+H : Switch into house mode.<br>'
        help_text += 'In house mode you can select <b>four</b> lines in the Plan View to create a house. '
        help_text += 'The first line indicates the front facade of the house. '
        help_text += 'The attributes of the house can be modified in the Attributes Control widget (bottom).</li>'
        help_text += '<li>Ctrl+W : Switch into Inca wall mode.<br>'
        help_text += 'In Inca wall mode you can select <b>one</b> line in the Plan View to create a wall. '
        help_text += 'The attributes of the wall can be modified in the Attributes Control widget.</li>'
        help_text += '<li>Ctrl+T : Switch into terrace mode.<br>'
        help_text += 'In terrace mode, you must first select <b>two</b> lines and enter <b>"s"</b> to create stairs. '
        help_text += 'The first line indicates the length of the stairs (from high point to low point). '
        help_text += 'The second line indicates the width of the stairs. '
        help_text += 'You can then select <b>at least two</b> lines and enter <b>"t"</b> to create terraces. '
        help_text += 'The height of the terraces depend on the stairs, therefore the start points of the lines '
        help_text += 'must lie within the area of the stairs. </li>'
        help_text += '<li>Ctrl+R : Generate a plan randomly.</li></ul>'

        help_text += '<b>Site Plan View</b> (left)<br>'
        help_text += 'You can draw lines to create houses, walls, stairs, and terraces by pressing and releasing '
        help_text += 'the mouse. Houses are outlined red. Walls are outlined green. Terraces are outlined blue. <br>'
        help_text += 'Shortcuts:'
        help_text += '<ul><li>s : to create stairs. This is used in terrace mode after two lines are selected.</li>'
        help_text += '<li>t : to create terraces. This is used in terrace mode after several lines are selected.</li>'
        help_text += '<li>c : to clear the selected lines. </li>'
        help_text += '<li>r : to reset the model. </li></ul>'

        help_text += '<b>OpenGL Renderder View</b> (Right)<br>'
        help_text += 'Use left mouse button to translate the model. Use mouse wheels to zoom the model. '
        help_text += 'Use mouse middle button to rotate the model.<br>'
        self.manual_box.setText(help_text)

    def update(self):
        if self.mode == HOUSE:
            self.tab.clear()
            for house in self.model.generated_houses:
                hcontrol = HouseControlView(self.model, house)
                hcontrol.setMinimumSize(960, 130)
                hcontrol.setMaximumSize(960, 130)
                self.tab.addTab(hcontrol, "House " + str(house.id))

        if self.mode == INCAWALL:
            self.tab.clear()
            for wall in self.model.generated_walls:
                wcontrol = WallControlView(self.model, wall)
                wcontrol.setMinimumSize(960, 130)
                wcontrol.setMaximumSize(960, 130)
                self.tab.addTab(wcontrol, "Wall" + str(wall.id))

        if self.mode == TERRACE:
            self.tab.clear()

    def slotFile(self):
        filename = QtGui.QFileDialog.getOpenFileName()
        if filename <> '':
            image = QtGui.QPixmap(filename)
            image_size = image.size()
            image_item = QtGui.QGraphicsPixmapItem(image)
            if self.plan_image:
                self.scene_view.scene.removeItem(self.plan_image)
            self.plan_image = image_item
            self.scene_view.load_image(self.plan_image, image_size)

    def slotHouseMode(self):
        self.hmode.setChecked(True)
        self.wmode.setChecked(False)
        self.tmode.setChecked(False)
        self.mode = HOUSE
        self.update()
        self.statusBar().showMessage('House Mode')

    def slotWallMode(self):
        self.wmode.setChecked(True)
        self.hmode.setChecked(False)
        self.tmode.setChecked(False)
        self.mode = INCAWALL
        self.update()
        self.statusBar().showMessage('Wall Mode')

    def slotTerraceMode(self):
        self.tmode.setChecked(True)
        self.wmode.setChecked(False)
        self.hmode.setChecked(False)
        self.mode = TERRACE
        self.update()
        self.statusBar().showMessage('Terrace Mode')

    def slotGenerate(self):
        self.model.random_generate()

    def slotManual(self):
        self.manual_box.show()

if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec_())
