#!/usr/bin/python

from random import *
from math import *
from OpenGL import GL
from PyQt4 import QtGui
from ground import Ground
from house import House
from utils import *

from attributes import *
HA = HouseAttributes()

class IncaModel():
    def __init__(self):
        self.views = []
        self.houses = []
        self.generated_houses = []
        self.walls = []
        self.generated_walls = []
        self.ground = None

    def register_view(self, view):
        self.views.append(view)

    def generate(self):
        for house in self.houses:
            house.generate()
            self.generated_houses.append(house)

        for wall in self.walls:
            wall.generate()
            self.generated_walls.append(wall)

        self.houses = []
        self.walls = []

        for view in self.views:
            view.update()

    def random_generate(self):
        self.clear()
        self.ground = Ground(600, 400)

        # one elevation
        start = (randint(150, 180), randint(10, 50))
        if random() > 0.5:
            end = (start[0] + randint(20, 40), start[1] + randint(250, 350))
        else:
            end = (start[0] - randint(20, 40), start[1] + randint(250, 350))
        angle = atan2(end[0] - start[0], end[1] - start[1]) 
        self.ground.elevation(-angle, start, end)
        self.update()

        k = 1.0 / tan(angle)
        b = start[1] - k * start[0] - 10

        l = randint(3, 6)
        llines = []
        dlx = (end[0] - start[0]) / float(l)
        dly = (end[1] - start[1]) / float(l)
        for i in xrange(l):
            # calculate start point and end point for the line
            x1 = start[0] + dlx * i
            y1 = k * x1 + b #start[1] + dly * i
            x2 = x1 - 100
            y2 = y1 + randint(-10, 10)
            llines.append(QtGui.QGraphicsLineItem(x1, y1, x2, y2))
        llines.append(QtGui.QGraphicsLineItem(0, 400, 600, 400))
        self.ground.terrace(llines)
        self.update()

        r = randint(2, 4)
        rlines = []
        drx = (end[0] - start[0]) / float(r)
        dry = (end[1] - start[1]) / float(r)
        for i in xrange(r):
            x1 = start[0] + drx * i
            y1 = k * x1 + b #start[1] + dry * i
            x2 = x1 + 100
            y2 = y1 + randint(-5, 5)
            rlines.append(QtGui.QGraphicsLineItem(x1, y1, x2, y2))
        rlines.append(QtGui.QGraphicsLineItem(0, 400, 600, 400))
        self.ground.terrace(rlines)
        self.update()

        endx = 580
        if (random() < 0.5):
            endx = 500
            astart = (randint(550, 570), randint(10, 50))
            if random() >= 0.5:
                aend = (astart[0] + randint(20, 30), astart[1] + randint(250, 350))
            else:
                aend = (astart[0] - randint(20, 30), astart[1] + randint(250, 350))
            aangle = atan2(aend[0] - astart[0], aend[1] - astart[1]) 
            self.ground.elevation(-aangle, astart, aend)
            self.update()

        # generate houses
        if end[0] > start[0]:
            startx = end[0] + (end[0] - start[0]) * cos(angle) + 20
        else:
            startx = start[0] + 20
        max_depth = 0.4 * dry
        for i in xrange(r):
            uy = rlines[i].line().y1()
            num_houses = randint(2, 4)
            dx = (endx - startx) / num_houses
            max_width = 0.6 * dx

            for j in xrange(int(num_houses)):
                hlines = []
                x1 = startx + dx * j
                x2 = startx + dx * j + max_width
                y1 = uy + 0.5 * dry + 0.5 * max_depth
                y2 = uy + 0.5 * dry - 0.5 * max_depth
                hlines.append(QtGui.QGraphicsLineItem(x1, y1, x2, y1))
                hlines.append(QtGui.QGraphicsLineItem(x2, y1, x2, y2))
                hlines.append(QtGui.QGraphicsLineItem(x2, y2, x1, y2))
                hlines.append(QtGui.QGraphicsLineItem(x1, y2, x1, y1))
                
                attr = HA.generate_attributes(hlines)
                h = House(attr)
                h.set_translation(comp_house_trans(hlines, 600, 400, self))
                h.set_rotation(uniform(-10, 10))
                self.add_house(h)
                self.update()


    def clear(self):
        if self.ground:
            self.ground = Ground(self.ground.width, self.ground.length)
        self.houses = []
        self.generated_houses = []
        self.walls = []
        self.generated_walls = []

    def render(self):
        if self.ground != None:
            self.ground.render()

        for house in self.generated_houses:
            GL.glPushMatrix()
            house.render()
            GL.glPopMatrix()

        for wall in self.generated_walls:
            GL.glPushMatrix()
            wall.render()
            GL.glPopMatrix()

    def update(self):
        self.generate()
        self.render()

    def ground_size(self, width, height):
        self.ground = Ground(width, height)

    def add_house(self, house):
        house.set_id(len(self.generated_houses))
        self.houses.append(house)
        self.update()

    def update_house(self, house):
        house.generate()
        for view in self.views:
            view.update()

        self.render()
        
    def add_wall(self, wall):
        wall.set_id(len(self.generated_walls))
        self.walls.append(wall)
        self.update()

    def update_wall(self, wall):
        try:
            self.generated_wall.remove(wall)
        except:
            pass
        self.walls.append(wall)
        self.update()

    def elevation(self, angle, start, end):
        self.ground.elevation(angle, start, end)
        self.generate()

    def terrace(self, lines):
        self.ground.terrace(lines)
        self.generate()

    def get_height(self, x, y):
        return self.ground.get_height(x, y)
